#!/usr/bin/perl -w
# J. C. Lansing
# 05/03/05
# Script to extract traces from an Xmgr plot
use strict;

my $i;
my @keep;
my $line;

foreach $i (@ARGV) {
	if ($i eq "-h") {
		while ($line = <DATA>) {
			print "$line";
		}
		exit(0);
	} else {
		push(@keep, $i);
	}
}
my $pl;
my @head;
my @plot;
my @label;
my %label;
while ($line = <STDIN>) {
	chomp($line);
	if (!defined($pl)) {
		if ($line =~ /^\@\s+S(\d+)\s/i) {
			$pl = $1;
			push(@{$plot[$pl]}, $line);
		} else {
			push(@head, $line);
		}
	} else {
		if ($line =~ /^\s*&/) {
			push(@{$plot[$pl]}, $line);
			undef($pl);
		} elsif ($line =~ /^\@\s*legend\s.*"\S+\s+(\S+)"/i) {
			push(@{$plot[$pl]}, $line);
			$label[$pl] = $1;
			$label{$1} = 1;
		} else {
			push(@{$plot[$pl]}, $line);
		}
	}
}
my @select;
my $n;
foreach $n (@keep) {
	for ($i = 0; $i <= $#plot; $i++) {
		if ($label[$i] =~ /$n/i) {
			push(@select, $i);
		}
	}
}

if ($#select < 0) {
	print STDERR "No matches to plot labels.\n";
	print STDERR "Labels in plot:";
	for ($i = 0; $i <= $#plot; $i++) {
		$label{$label[$i]} = 1;
	}
	foreach $i (sort(keys(%label))) {
		print STDERR "  $i";
	}
	print STDERR "\n";
} else {
	foreach $line (@head) { print "$line\n"; }
	my $num = 0;
	foreach $i (@select) {
		foreach $line (@{$plot[$i]}) { 
			my $color = int($num / 2) + 1;
			$line =~ s/(\sS)(\d+)(\b)/$1$num$3/;
			$line =~ s/(\slegend\sstring\s)(\d+)(\b)/$1$num$3/;
			$line =~ s/(\ss\d+\s+color\s+)(\d+)(\b)/$1$color$3/i;
			$line =~ s/(\ss\d+\s+symbol\s+color\s+)(\d+)(\b)/$1$color$3/i;
			print "$line\n"; 
		}
		++$num;
	}
}

__DATA__
extract_xmgr.pl -- Script to extract only some traces in an Xmgr plot
Usage:  extract_xmgr.pl {plot labels} < input.xmgr > output.xmgr

