#!/usr/bin/perl -w
# J. C. Lansing
# 02/10/05


sub min {
	my $list = $_[0];
	my $i;
	my $min = $$list[0];
	for ($i = 1; $i <= $#$list; $i++) {
		if ($$list[$i] < $min) { $min = $$list[$i]; }
	}
	return $min;
}

sub max {
	my $list = $_[0];
	my $i;
	my $max = $$list[0];
	for ($i = 1; $i <= $#$list; $i++) {
		if ($$list[$i] > $max) { $max = $$list[$i]; }
	}
	return $max;
}

sub plot {
	my $title = $_[0];
	my $x = $_[1];
	my $y = $_[2];
	my $dy = $_[3];
	my $i;
	my $up;
	my $dn;
	open(OUT,">$title.xmgr");
	print OUT "@ CLEAR STRING\n";
	print OUT "@ VIEW XMIN 0.20\n";
	print OUT "@ VIEW XMAX 0.90\n";
	print OUT "@ VIEW YMIN 0.35\n";
	print OUT "@ VIEW YMAX 0.85\n";
	print OUT "@ XAXIS LABEL \"1/\\xt\\f{}\\sCP\\N (s\\S-1\\N)\"\n";
	print OUT "@ YAXIS LABEL \"\\qR\\Q\\s2\\N\\Seff\\N (s\\S-1\\N)\"\n";
	print OUT "@ XAXIS TICKLABEL FORMAT DECIMAL\n";
	print OUT "@ YAXIS TICKLABEL FORMAT DECIMAL\n";
	print OUT "@ XAXIS TICKLABEL CHAR SIZE 0.8\n";
	print OUT "@ YAXIS TICKLABEL CHAR SIZE 0.8\n";
	print OUT "@ XAXIS TICKLABEL PREC 0\n";
	print OUT "@ YAXIS TICKLABEL PREC 0\n";
	$up = &max($x) + 100.0;
	print OUT "@ WORLD XMAX $up\n";
	print OUT "@ WORLD XMIN 0.0\n";
	$up = &max($y) + 2*&max($dy);
	$dn = &min($y) - 2*&max($dy);
	print OUT "@ WORLD YMAX $up\n";
	print OUT "@ WORLD YMIN $dn\n";
	print OUT "@ XAXIS TICK MAJOR 500.000\n";
	print OUT "@ XAXIS TICK MINOR 250.000\n";
	print OUT "@ YAXIS TICK MAJOR 5.000\n";
	print OUT "@ YAXIS TICK MINOR 2.500\n";
	print OUT "@ with g0\n";
	print OUT "@ g0 on\n";
	print OUT "@ g0 label off\n";
	print OUT "@ g0 hidden false\n";
	print OUT "@ TITLE \"$title\"\n";
	print OUT "@ S0 LINESTYLE 0\n";
	print OUT "@ S0 COLOR 1\n";
	print OUT "@ S0 SYMBOL 2\n";
	print OUT "@ S0 SYMBOL SIZE 0.5\n";
	print OUT "@ S0 SYMBOL FILL 1\n";
	print OUT "@ S0 SYMBOL COLOR 1\n";
	print OUT "@ S0 ERRORBAR LENGTH 0.5\n";
	print OUT "@ TARGET S0\n";
	print OUT "@ TYPE xydy\n";
	for ($i = 0; $i <= $#{$x}; $i++) {
		print OUT "  $$x[$i]\t$$y[$i]\t$$dy[$i]\n";
	}
	print OUT "&\n";
	close(OUT);
}

sub help {
	my @e;
	my $exe = (caller(0))[1];
	@e = split(/\//, $exe);
	$exe = $e[$#e];
	print "Script for creation of Xmgr plots from a GLOVE input file.\n";
	print "Usage: $exe < glove_input_file\n";
	exit(0);
}


# ------------ Start of main program ------------

if ($#ARGV >= 0) { &help; }

my ($line, $title);
my (@e, @x, @y, @dy);

while ($line = <STDIN>) {
	chomp($line);
	if ($line =~ /^\s*title\s+(\S+)/i) {
		if (defined($title)) {
			# Print out the previous graph
			&plot($title,\@x,\@y,\@dy);
		}
		$title = $1;
		undef(@x);
		undef(@y);
		undef(@dy);
	} elsif (defined($title) && $line =~ /^\s*data\s+(\S+)\s.*\s(\S+)\s+(\S+)\s*$/i) {
		push(@x,$1);
		push(@y,$2);
		push(@dy,$3);
	}
}
if (defined($title)) {
	&plot($title,\@x,\@y,\@dy);
}



