// Xmgr class
//	Object to keep track of XMGR plot instructions
// Kenji Sugase		5/8/2012

#ifndef __XMGR_H__
#define __XMGR_H__

#include <iostream>
#include <string>
#include <cmath>
#include <cfloat>
#include <iomanip>
#include "FuncLib.h"
#include "ParSet.h"
#include "ParDef.h"
#include "Data.h"
#include "Type.h"

struct Instr
{
	string label;		// Title of the plot
	dvect x;		// Use this X for plotting		
	ivect idx;		// Use this set of idx for plotting
	double minPlot;
	double maxPlot;
	int nPlot;
	double slope;

	Instr(const string &tag, const dvect &data, const ivect &index, double min, double max, int size, double option)
		: label(tag), x(data), idx(index), minPlot(min), maxPlot(max), nPlot(size), slope(option) {}
};

class Xmgr
{
private:
	vector<Instr> instr;
	svect command;
	bool fXmgr;
	bool fError;

	void headerStyle(ostream &xout, const string &title, const string &eqn) const;
	void plotStyle(ostream &xout, int np, int color, const string &label) const;
	void lineStyle(ostream &xout, int np, int color, const string &label) const;
	void defaultString(ostream &xout) const;
	bool checkIndex(const ivect &plot_index, const i2vect &parm_index) const;

public:
	Xmgr();
	void SetXmgr(bool flag);
	void SetError(bool flag);
	bool isXmgr() const;
	bool isError() const;
	void header(const string &type);
	void add_XMGRcommand(const char *line);
	void add_plot(const string &label, const dvect &dataX, const ivect &index
		, double min = DBL_MAX, double max = -DBL_MAX, int step = 0, double option = 1);
	void xplot(ostream &xout, const ParSet &pG, size_t ns, const ParDef &parm
		, const Funclib &func) const;
	void annot_plot(ostream &xout, const ParSet &pG, size_t ns, const ParDef &parm
		, const dvect *err = NULL) const;
	void annot_sims(ostream &xout, const ParSet &pM, const ParSet &pS, size_t ns
		, const ParDef &parm, int nsim, const string &label = "Sim") const;
};

#endif
