// func_exp.cpp
// Kenji Sugase		2/21/2012

#include <iostream>
#include <cmath>
#include <cstdlib>
#include "Data.h"
#include "FuncLib.h"
#include "Type.h"
#include "Mathlib.h"

using namespace std;


// EXP4: f(x) = Aexp(Bx) + C
// par[0]	A	(unitless)
// par[1]	B	(unitless)
// par[2]	C	(unitless)
double f2exp3(const Data &data, double *par, dvect &dy, const bvect &fixed)
{
	int i_a = data.map[0];
	int i_b = data.map[1];
	int i_e = data.map[2];
	int i_f = data.map[3];
	
	double x, A, AA, Bx, expBx;
	double Ex, expEx, F;
	x  = data.x[0];
	A  = par[0];
	Bx = x * par[1];
	Ex = x * par[2];
	F  = par[3];
	if (Bx < EXP_MIN) { return -HUGE_VAL; }
	if (Ex < EXP_MIN) { return -HUGE_VAL; }
	if (Bx > EXP_MAX) { return  HUGE_VAL; }
	if (Ex > EXP_MAX) { return  HUGE_VAL; }
	expBx = exp(Bx);
	expEx = exp(Ex);
        AA = abs (1-A);
	if (dy.size()) {
		dy[i_a] = exp(Bx);
		dy[i_b] = A * x * exp(Bx);
		dy[i_e] = AA * x * exp(Ex);
		dy[i_f] = 1;
	}
	return A * expBx + AA * expEx + F;
}

// EXP3: f(x) = M0 - 2*M0*x*exp(Bx) 
// par[0]       M0      (unitless)
// par[1]       B       (unitless)
double fsire(const Data &data, double *par, dvect &dy, const bvect &fixed)
{
        int i_a = data.map[0];
        int i_b = data.map[1];

        double x, M0, Bx, expBx;
        x  = data.x[0];
        M0  = par[0];
        Bx = x * par[1];
        if (Bx < EXP_MIN) { return -HUGE_VAL; }
        if (Bx > EXP_MAX) { return  HUGE_VAL; }
        expBx = exp(Bx);
        if (dy.size()) {
                dy[i_a] = 1-2*exp(Bx);
                dy[i_b] = -2 * M0 * x * exp(Bx);
        }
        return M0 - 2 * M0 * expBx;
}


int Funclib::Read_Bio_Funcs(int idx)
{
	// Exponential function with a constant offset (3 parameters)
	idx = Define_Func(&f2exp3, "2Exp3", "f(x) = Aexp(Bx) +  (1-A)exp(Ex) + F", "EXP");
	Define_Parm(idx, "a"    ,-1.0e9, 1.0e9);
	Define_Parm(idx, "b"    ,-1.0e6, 1.0e6);
	Define_Parm(idx, "e"    ,-1.0e6, 1.0e6);
	Define_Parm(idx, "f"    ,-1.0e9, 1.0e9);

   // SIRE T1 function with a constant offset (2 parameters)
        idx = Define_Func(&fsire, "SIRE", "f(x) = M0-2*Mo*exp(Bx)", "EXP");
        Define_Parm(idx, "a"    ,-1.5e0, 1.5e0);
        Define_Parm(idx, "b"    ,-1.0e6, 1.0e6);


	return idx;
}



