// ParDef class
//	Class to hold temporary info about global variables
// Kenji Sugase		4/1/2013

#include <cstdlib>
#include "ParDef.h"

using namespace std;

// Class Elem methods
Elem::Elem() : parm_type(UNDEF), grid_start(-HUGE_VAL), grid_end(HUGE_VAL)
	, grid_range(0), grid_step(0), grid_size(1), grid_interval(GRID_NORMAL) {}

void Elem::setName(const string &PAR, const string &SET)
{
	parm_name     = PAR;
	parm_fullname = SET + "::" + PAR;
}

void Elem::setGrid(double start, double end, int size, double interval, double initial)
{
	parm_type     = GRID;
	grid_start    = start;
	grid_end      = end;
	grid_range    = grid_end - grid_start;
	grid_size     = size;
	grid_interval = interval;
	grid_step     = 0;
	if (grid_size > 1){
		switch (grid_interval) {
		case GRID_NORMAL:
			grid_step = grid_range / (size - 1);
			break;
		case GRID_EXP:
			grid_step = log(grid_range) / (size - 1);
			break;
		case GRID_GAUSS:
			grid_step = sqrt(log(grid_range)) / (size - 1);
			break;
		case GRID_TENTH:
			grid_step = log10(grid_range) / (size - 1);
			break;
		case GRID_iEXP:
			grid_step = log(grid_range) / (size - 1);
			break;
		case GRID_iGAUSS:
			grid_step = sqrt(log(grid_range)) / (size - 1);
			break;
		case GRID_iTENTH:
			grid_step = log10(grid_range) / (size - 1);
			break;
		}
	}
	grid_initial = start;
	if (std::isfinite(initial)) {
		grid_initial = initial;
	}
}

void Elem::setFixed(double value)
{
	parm_type    = FIXED;
	grid_start   = value;
	grid_end     = value;
	grid_initial = value;
}

void Elem::Restraint(double start, double end)
{
	grid_lower = start;
	grid_upper = end;
}

void Elem::Add_nSetByPAR(int num)
{
	nSetByPAR.push_back(num);
}

bool Elem::IsLocalNonFIXED(int ns) const
{
	return (nSetByPAR.size() == 1 && nSetByPAR[0] == ns && parm_type == GRID);
}

bool Elem::isLocal() const
{
	return (parm_name != "GLOBAL");
}

bool Elem::isGlobal() const
{
	return (nSetByPAR.size() >= 2 && parm_type == GRID);
}

void ParDef::setRule(const Rules &ptr)
{
	rule = ptr;
}

// add a regular global parameter
void ParDef::Add_GPAR(const string &name, double start, double end, int size, double interval, double initial)
{
	// Make a new parameter entry
	int np = pv.size();
	pv.resize(np+1);
	pv[np].setName(name, "GLOBAL");
	pv[np].setGrid(start, end, size, interval, initial);

	// Make a new global entry
	gPar.push_back(Global(np, name, false));
	fixed.push_back(false);
	allPAR.push_back(np);
	allGPAR.push_back(np);
}

// add a restratined global parameter
void ParDef::Add_GREST(const string &name, double start, double end, int size, double interval, double initial)
{
	// Make a new parameter entry
	int np = pv.size();
	pv.resize(np+1);
	pv[np].setName(name, "GLOBAL");
	pv[np].setGrid(start, end, size, interval, initial);
	pv[np].Restraint(start, end);

	// Make a new global entry
	gPar.push_back(Global(np, name, true));
	fixed.push_back(false);
	allPAR.push_back(np);
	allGPAR.push_back(np);
}

// add a fixed global parameter
void ParDef::Add_GFIXED(const string &name, double value)
{
	// Make a new parameter entry
	int np = pv.size();
	pv.resize(np+1);
	pv[np].setName(name, "GLOBAL");
	pv[np].setFixed(value);

	// Make a new global entry
	gPar.push_back(Global(np, name, false));
	fixed.push_back(true);
	allPAR.push_back(np);
	allGPAR.push_back(np);
}

// Add a local parameter
void ParDef::Add_PAR(const string &name, double start, double end, int size, double interval, double initial)
{
	// Make a new parameter entry
	int np = pv.size();
	pv.resize(np+1);
	pv[np].setName(name, title[nSet]);
	pv[np].setGrid(start, end, size, interval, initial);

	// Make a new local entry
	ivect index;					// index defined in the input file
	int nbase = rule.decompose(name, index);	// base parameter number
	lPar[nSet].push_back(Local(np, name, nbase, index));
	fixed.push_back(false);

	double min = rule.Get_LowerBound(nbase);
	double max = rule.Get_UpperBound(nbase);
	pv[np].Restraint(min, max);

	// add parameter map
	Add_ParmMap(np, name);
	allPAR.push_back(np);
}

// Tie a PAR parameter to a GPAR parameter
void ParDef::Add_PAR(const string &name, string &GPAR)
{
	// Find the global parameter entry
	int np = Find_GPAR(name, GPAR);

	// Make a new local entry
	ivect index;					// index defined in the input file
	int nbase = rule.decompose(name, index);	// base parameter number
	lPar[nSet].push_back(Local(np, name, nbase, index));

	if (!gPar[np].restraint) {
		double min = rule.Get_LowerBound(nbase);
		double max = rule.Get_UpperBound(nbase);
		pv[np].Restraint(min, max);
	}

	// add parameter map
	Add_ParmMap(np, name);
}

// Add a local parameter
void ParDef::Add_REST(const string &name, double start, double end, int size, double interval, double initial)
{
	// Make a new parameter entry
	int np = pv.size();
	pv.resize(np+1);
	pv[np].setName(name, title[nSet]);
	if (std::isfinite(end)) {
		pv[np].setGrid(start, end, size, interval, initial);
	} else {
		pv[np].setGrid(start, start, 1, interval, start);
	}

	// Make a new local entry
	ivect index;					// index defined in the input file
	int nbase = rule.decompose(name, index);	// base parameter number
	lPar[nSet].push_back(Local(np, name, nbase, index));
	fixed.push_back(false);

	pv[np].Restraint(start, end);

	// add parameter map
	Add_ParmMap(np, name);
	allPAR.push_back(np);
}

// Add a FIXED parameter
void ParDef::Add_FIXED(const string &name, double start)
{
	// Make a new parameter entry
	int np = pv.size();
	pv.resize(np+1);
	pv[np].setName(name, title[nSet]);
	pv[np].setFixed(start);

	// Make a new local entry
	ivect index;					// index defined in the input file
	int nbase = rule.decompose(name, index);	// base parameter number
	lPar[nSet].push_back(Local(np, name, nbase, index));
	fixed.push_back(true);
	double min = rule.Get_LowerBound(nbase);
	double max = rule.Get_UpperBound(nbase);
	pv[np].Restraint(min, max);

	// add parameter map
	Add_ParmMap(np, name);
	allPAR.push_back(np);
}

// add parameter map
void ParDef::Add_ParmMap(int np, const string &name)
{
	nParBySet[nSet].push_back(np);
	ParNameBySet[nSet].push_back(name);
}

int ParDef::Find_GPAR(const string &name, const string &GPAR)
{
	// Find the global parameter entry
	int szGbl = gPar.size();
	for (int i = 0; i < szGbl; i++) {
		if (gPar[i].name == GPAR) { return i; }
	}
	
	// Global parameter entry was not found
	cerr << "ERROR: Set \'" << title[nSet] << "\' par \'" << name
	     << "\' references unknown variable \'" << GPAR << "\'\n";
	exit(1);
}

// add a new data set
// this function is called after adding all parameters and data of each dataset
void ParDef::Add_Set(const string &name)
{
	if (title.size()) {
		if (Calc_lDoF()) {
			Store_SET();
		} else {
			Remove_SET();
		}
	}
	// make a new entry
	nSet = SETsize();
	title.push_back(name);
	lPar.resize(nSet+1);
	nDataBySet.resize(nSet+1);
	nParBySet.resize(nSet+1);
	ParNameBySet.resize(nSet+1);
}

// remove a bad dataset
void ParDef::Remove_SET()
{
	title.pop_back();
	ldof.pop_back();
	lPar.pop_back();
	// remove parameters
	int sz = nParBySet[nSet].size();
	for (int np = 0; np < sz; np++) {
		if (pv[nParBySet[nSet][np]].isLocal()) {
			pv.pop_back();
			allPAR.pop_back();
			fixed.pop_back();
		}
	}
	nParBySet.pop_back();
	ParNameBySet.pop_back();
	// remove data
	sz = nDataBySet[nSet].size();
	for (int nd = 0; nd < sz; nd++) {
		data.pop_back();
		allData.pop_back();
	}
	nDataBySet.pop_back();
}

// store the title/number/parameters for each dataset
void ParDef::Store_SET()
{
	int sz = lPar[nSet].size();
	for (int i = 0; i < sz; i++) {
		int np = lPar[nSet][i].num;
		pv[np].Add_nSetByPAR(nSet);
		pv[np].nBasePar = lPar[nSet][i].nBasePar;
	}
}

// add a data point
void ParDef::Add_DATA(dvect &condition, ivect &index, double value, double error)
{
	ivect PARs = Get_nPARs(index);		// PAR numbers used for fitting this DATA
	int nDATA = data.size();		// DATA number
	Data onedata(nSet, condition, index, value, error, PARs);
	data.push_back(onedata);

	nDataBySet[nSet].push_back(nDATA);
	allData.push_back(nDATA);
}

ivect ParDef::Get_nPARs(const ivect &index)
{
	ivect PARs;
	int szbPAR = rule.basePAR_size();	// the number of base paramters
	PARs.resize(szbPAR);
	int j;
	for (int i = 0; i < szbPAR; i++) {
		// Find the match
		if ((j = match_local(i, index)) == -1) {
			cerr << "ERROR: Set \'" << title[nSet] << "\' missing declaration of";
			string name;
			rule.markup(name, i, index);
			cerr << "parameter \'" << name << "\'\n";
			exit(1);
		}
		PARs[i] = lPar[nSet][j].num;
	}
	return PARs;
}

void ParDef::Get_datamap(Data &data, size_t ns)
{
	int sz = rule.basePAR_size();
	data.map.resize(sz);
	int j;
	for (int i = 0; i < sz; i++) {
		if ((j = match_local(i, ns, data.idx)) >= 0) {
			data.map[i] = nParBySet[ns][j];
		}
	}
}

int ParDef::match_local(int nb, const ivect &index)
{
	int sz = lPar[nSet].size();
	for (int i = 0; i < sz; i++) {
		if (lPar[nSet][i].isMatch(nb, index)) { return i; }
	}
	return -1;
}

int ParDef::match_local(int nb, size_t ns, const ivect &index)
{
	int sz = lPar[ns].size();
	for (int i = 0; i < sz; i++) {
		if (lPar[ns][i].isMatch(nb, index)) { return i; }
	}
	return -1;
}

// Prepare parameter map
void ParDef::prep()
{
	// store the last dataset
	if (Calc_lDoF()) { Store_SET();  }
	else             { Remove_SET(); }
	nSet = SETsize();

	// calculate degrees of freedom
	Calc_gDoF();
	int szPAR = PAR_size();
	mask_global.resize(szPAR);
	for (int np = 0; np < szPAR; np++) {
		mask_global[np] = pv[np].isGlobal();
	}
	mask_local.resize(nSet);
	for (int ns = 0; ns < nSet; ns++) {
		mask_local[ns].resize(szPAR);
		for (int np = 0; np < szPAR; np++) {
			mask_local[ns][np] = pv[np].IsLocalNonFIXED(ns);
		}
	}

	// store index numbers used in each dataset  	
	indexBySet.resize(nSet);
	for (int ns = 0; ns < nSet; ns++) {
		int szd = nDataBySet[ns].size();
		for (int nd = 0; nd < szd; nd++) {
			bool match = false;
			int szi = indexBySet[ns].size();
			for (int ni = 0; ni < szi; ni++) {
				if (indexBySet[ns][ni] == data[nDataBySet[ns][nd]].idx) {
					match = true;
					break;
				}
			}
			if (!match) {
				indexBySet[ns].push_back(data[nDataBySet[ns][nd]].idx);
			}
		}
	}
}

// calculate local degrees of freedom
bool ParDef::Calc_lDoF()
{
	// add the number of data points
	ldof.push_back(nDataBySet[nSet].size());
	// substract the number of fitting parameters
	int sz = nParBySet[nSet].size();
	for (int i = 0; i < sz; i++) {
		int np = nParBySet[nSet][i];
		if (!isFIXED(np)) { --ldof[nSet]; }
	}
	if (ldof[nSet] <= 0) {
		cout << "WARNING: Too few data points. '" << title[nSet] << "' is omitted.\n";
		return false;
	}
	return true;
}

// calculate global degrees of freedom
void ParDef::Calc_gDoF()
{
	gdof = data.size();	// global degrees of freedom
	int szg = pv.size();
	for (int i = 0; i < szg ; i++) {
		if (!fixed[i]) { --gdof; }
	}
	if (gdof <= 0) {
		cout << "#ERROR: More adjustable parameters than data points.\n";
		exit(1);
	}
}

void ParDef::setgDoF(int val)
{
	gdof = val;
}

void ParDef::setlDoF(size_t n, int val)
{
	ldof[n] = val;
}

int ParDef::SETsize() const
{
	return title.size();
}

int ParDef::PAR_size() const
{
	return pv.size();
}

int ParDef::gDoF() const
{
	return gdof;
}

int ParDef::lDoF(size_t n) const
{
	return ldof[n];
}

const string & ParDef::TITLE(size_t n) const
{
	return title[n];
}

const string & ParDef::Name(size_t n) const
{
	return pv[n].parm_name;
}

const string & ParDef::FullName(size_t n) const
{
	return pv[n].parm_fullname;
}

double ParDef::grd_start(size_t n) const
{
	return pv[n].grid_start;
}

double ParDef::grd_end(size_t n) const
{
	return pv[n].grid_end;
}

double ParDef::grd_initial(size_t n) const
{
	return pv[n].grid_initial;
}

double ParDef::grd_range(size_t n) const
{
	return pv[n].grid_range;
}

double ParDef::grd_step(size_t n) const
{
	return pv[n].grid_step;
}

int ParDef::grd_size(size_t n) const
{
	return pv[n].grid_size;
}

int ParDef::grd_interval(size_t n) const
{
	return pv[n].grid_interval;
}

bool ParDef::isGRID(size_t n) const
{
	return pv[n].parm_type == GRID;
}

bool ParDef::isFIXED(size_t n) const
{
	return pv[n].parm_type == FIXED;
}
bool ParDef::isLegal(size_t n, double val) const
{
	return (pv[n].grid_lower <= val && val <= pv[n].grid_upper);
}

bool ParDef::isGlobal(size_t n) const
{
	return pv[n].isGlobal();
}

bool ParDef::isLocal(size_t n) const
{
	return !pv[n].isGlobal();
}
