// Calc class		Object to keep track of calculation instructions
// Kenji Sugase		9/20/2012

#ifndef __CALC_H__
#define __CALC_H__


#include "FuncLib.h"
#include "Data.h"
#include "Type.h"

class OneCalc {
private:
	int calc_type;
	string calc_label;
	ivect nsim;
	dvect sum;
	dvect sum2;
	dvect orig;

public:
	Data calc_data;

	OneCalc(int type, const string &label, const Data &data);
	void prep(size_t ns);
	void zero();
	void addVal(size_t ns, double val);
	void addJack(size_t ns, double val);
	double mean(size_t ns);
	double stdev(size_t ns);
	double jackStdev(size_t ns);
	void store(size_t ns, double val);
	int type() const;
	const string & label() const;
};

namespace {
	const int SIM_NONE    = 0;
	const int SIM_JACK    = 1;
	const int SIM_MONTE   = 2;
}

class Calc {
private:
	static const int CALC_UNDEF   = 0;
	static const int CALC_REX     = 1;
	static const int CALC_PB      = 2;
	static const int CALC_PB_RDC  = 3;
	static const int CALC_PB_MQ   = 4;

	vector<OneCalc> calc1;
	int type_jack;
	int type_Monte;
	int nMonte;

	bool checkIndex(const ivect &calc_index, const i2vect &parm_index);


public:
	Calc();
	void prep(int ns);
	void zero();
	void Add(const string &type, const string &label, const dvect &condition, const ivect &index);
	int size() const;
	const string & Label(size_t nc) const;
	double run(size_t nc, const dvect &p, pFunc fx, const i2vect &index);
	void addVal(size_t ns, size_t nc, double val);
	void addJack(size_t ns, size_t nc, double val);
	double calcMean(size_t ns, size_t nc);
	double calcStdev(size_t ns, size_t nc);
	double calcJackStdev(size_t ns, size_t nc);
	void storeResult(size_t ns, size_t nc, double val);
	Data & getData(size_t nc);
	void SetJack(int num);
	int getJackType() const;
	void SetMonte(int num, int iter);
	int getMonteType() const;
	int getMonteIter() const;
};

#endif
