// Mathlib class	Library of mathematic
// Kenji Sugase			4/1/2013

#ifndef __MATHLIB_H__
#define __MATHLIB_H__

#include <complex>
#include "Type.h"

namespace{
	const cdouble I(0, 1);

	const double EXP_MAX  = log(DBL_MAX);		// exp(EXP_MAX) is the largest double value
	const double EXP_MIN  = log(DBL_MIN);   	// exp(EXP_MIN) is the smallest double value
	const double COSH_MAX = acosh(DBL_MAX);		// cosh(COSH_MAX) is the largest double value
	const double COSH_MIN = 1.0;			// cosh(COSH_MIN) = 0

	template <class T> T sqr(T x)    { return x*x;     }
	template <class T> T cube(T x)   { return x*x*x;   }
	template <class T> T biquad(T x) { return x*x*x*x; }

	// a = A/2
	void solveQuadratic_real1(double a, double p, double q, double x, double *y, int i)
	{
		for (int j = 0; j < 2; j++) {
			if (j == 1) {
				p *= -1;
				q *= -1;
			}
			// calculate the discriminant D
			double t = a + p;
			double D = sqr(t)-4*(x + q);
			// evaluate the discriminant
			double m = -0.5*t;
			if (D == 0 ) {					// 2 equal roots
				y[  8*i+4*j] = m;
				y[2+8*i+4*j] = y[0+8*i+4*j];
				y[1+8*i+4*j] = 0;
				y[3+8*i+4*j] = 0;
			} else if (D > 0) {				// 2 equal roots
				double n = 0.5*sqrt(D);
				y[  8*i+4*j] = m + n;
				y[2+8*i+4*j] = m - n;
				y[1+8*i+4*j] = 0;
				y[3+8*i+4*j] = 0;
			} else {	// (D < 0)			// 2 complex roots
				y[  8*i+4*j] = m;
				y[2+8*i+4*j] = y[0+8*i+4*j];
				y[1+8*i+4*j] =  0.5*sqrt(-D);
				y[3+8*i+4*j] = -0.5*sqrt(-D);
			}
		}
	}
	
	// square root of complex (a + bi)
	void compRoot(double a, double b, double *zz)
	{
		double r = sqrt(sqr(a) + sqr(b));
		zz[0] = sqrt(0.5*( a + r));
		zz[1] = sqrt(0.5*(-a + r));
		if (b < 0) { zz[1] *= -1; }
	}
	
	// a = A/2
	void solveQuadratic_real2(double a, double p, double q, double x, double *y, int i)
	{
		double z[2];
		for (int j = 0 ; j < 2 ; j++ ) {
			if (j == 1) {
				p *= -1;
				q *= -1;
			}
			// calculate the discriminant D
			double u = sqr(a)-sqr(p)-4*x;				// real of D
			double v = 2*a*p-4*q;					// imag of D
			compRoot(u, v, z);
			y[  8*i+4*j] = 0.5*(-a + z[0]);
			y[1+8*i+4*j] = 0.5*(-p + z[1]);
			y[2+8*i+4*j] = 0.5*(-a - z[0]);
			y[3+8*i+4*j] = 0.5*(-p - z[1]);
		}
	}
	
	void solveQuadratic_comp(double a, const double *x, double *z1, double *z2, double *y, int k)
	{
		double z[2];
		for (int j = 0 ; j < 2 ; j++ ) {
			if (j == 1) {
				z1[0] *= -1;
				z1[1] *= -1;
				z2[0] *= -1;
				z2[1] *= -1;
			}
			// calculate the discriminant D
			double t = 0.5*a + z1[0];
			double u = sqr(t)-sqr(z1[1])-4*(x[1]+z2[0]);		// real of D
			double v = 2*(t)*z1[1]-4*(x[3]+z2[1]);			// imag of D
			compRoot(u, v, z);
			y[  8*k+4*j] = 0.5*(-t + z[0]);
			y[1+8*k+4*j] = 0.5*(-z1[1] + z[1]);
			y[2+8*k+4*j] = 0.5*(-t - z[0]);
			y[3+8*k+4*j] = 0.5*(-z1[1] - z[1]);
		}
	}
	
	double recal(double A, double B, double C, double D, double y, double yi)
	{
		double y2  = sqr(y);
		double y3  = cube(y);
		double y4  = biquad(y);
		if (yi == 0) {
			return y4 + A*y3 + B*y2 + C*y + D;		// residual
		}
		double yi2 = sqr(yi);
		double yi3 = cube(yi);
		double yi4 = biquad(yi);
		double real4 = y4 - 6*y2*yi2 + yi4;			//   y^4
		double real3 = A*(y3 - 3*y*yi2);			// A*y^3
		double real2 = B*(y2 - yi2);				// B*y^2
		double real1 = C*y;					// C*y
		double real  = real4 + real3 + real2 + real1 + D;
		double comp4 = 4*(y3*yi - y*yi3);			//   yi^4
		double comp3 = A*(3*y2*yi - yi3);			// A*yi^3
		double comp2 = B*2*y*yi;				// B*yi^2
		double comp1 = C*yi;					// C*yi
		double comp  = comp4 + comp3 + comp2 + comp1;
		return sqrt(sqr(real)+sqr(comp));			// residual
	}
	
	void solveQuartic_real(double A, double B, double C, double D, const double *x, double *y, double *residual)
	{
		for (int i = 0; i < 3; i++ ) {
			// determine p and q
			double pp = 0.25*sqr(A) + 2*x[i] - B;
			double qq = sqr(x[i]) - D;
			bool isComp = false;				// flag rr = 1 for real or -1 for complex in p and q
			if (pp < 0) {
				pp *= -1;
				qq *= -1;
				isComp = true;
			}
			double p = sqrt(pp);
			double q = sqrt(qq);
			double r = A*x[i] - C;				// = 2*p*q
			// sign check of p and q
			if ((r >= 0 && p*q < 0) || (r < 0 && p*q > 0)) { p *= -1; }
			if (isComp) { p *= -1; }
			// solve quadratic eauation
			// y^2 + (A/2-p)y + (x-q) = 0
			if (isComp) {
				solveQuadratic_real2(0.5*A, p, q, x[i], y, i);	// p and q are complex
			} else {
				solveQuadratic_real1(0.5*A, p, q, x[i], y, i);	// p and q are real
			}
			double w[4];
			for (int j = 0; j < 2; j++) {
				w[0+2*j] = recal(A, B, C, D, y[0+8*i+4*j], y[1+8*i+4*j]);
				w[1+2*j] = recal(A, B, C, D, y[2+8*i+4*j], y[3+8*i+4*j]);
			}
			residual[i] = w[0] + w[1] + w[2] + w[3];
			if (x[3] != 0) { return; }
		}
	}
	
	void solveQuartic_comp(double A, double B, double C, double D, const double *x, double *y, double *residual, int k)
	{
		// determine p and q
		double z1[2], z2[2];
		compRoot(sqr(A)/4+2*x[1]-B, 2*x[3], z1);		// p = sqrt(sqr(A)/4 + 2*(x[1]+x[3]*i) - B)
		compRoot(sqr(x[1])-sqr(x[3])-D, 2*x[1]*x[3], z2);	// q = sqrt(sqr(x[1]+x[3]*i) - D)
		double r = A*x[1] - C;					// real part of 2*p*q
		// sign check of p and q
		if ((r >= 0 && (z1[0]*z2[0]-z1[1]*z2[1]) < 0) || (r < 0 && (z1[0]*z2[0]-z1[1]*z2[1]) > 0)) {
			z1[0] *= -1;
			z1[1] *= -1;
		}
		// solve quadratic eauation
		// y^2 + (A/2-p)y + (x-q) = 0
		solveQuadratic_comp (A, x, z1, z2, y, k);
		double w[4];
		for (int j = 0; j < 2; j++) {
			w[0+2*j] = recal (A, B, C, D, y[0+8*k+4*j], y[1+8*k+4*j]);
			w[1+2*j] = recal (A, B, C, D, y[2+8*k+4*j], y[3+8*k+4*j]);
		}
		residual[k] = w[0] + w[1] + w[2] + w[3];
	}
	
	double solveCubic(double a, double b, double c, double d)
	{
		// calculate the discriminant D
		double M1 = b/(3*a);
		double M2 = c/a;
		double M3 = d/a;
		double p  = M2/3 - sqr(M1);
		double q  = 2*cube(M1) - M1*M2 + M3;
		double q2 = sqr(q);
		double p3 = cube(p);
		double D = q2 + 4*p3;
		// evaluate the discriminant
		if (p == 0 && q == 0 && D == 0) {
			// 3 equal roots
			return -cbrt(M3);
		} else if (D <= 0) {
			// 3 real roots
			double r = sqrt(-p3);
			double theta = acos(-q/(2*r));
			return 2*cbrt(r)*cos(theta/3) - M1;
		} else {	// D > 0
			// 1 real root and 2 complex roots
			double A = cbrt((-q+sqrt(D))/2);
			double B = cbrt((-q-sqrt(D))/2);
			return A + B - M1;
		}
	}
	
	void solve_cubic(double a, double b, double c, double d, double *x)
	{
		// calculate the discriminant D
		double M1 = b/(3*a);
		double M2 = c/a;
		double M3 = d/a;
		double p  = M2/3 - sqr(M1);
		double q  = 2*cube(M1) - M1*M2 + M3;
		double q2 = sqr(q);
		double p3 = cube(p);
		double D = q2 + 4*p3;
		// evaluate the discriminant
		if (p == 0 && q == 0 && D == 0) {
			// 3 equal roots
			x[0] = -cbrt(M3);
			x[1] = x[0];
			x[2] = x[1];
			x[3] = 0;
		} else if (D <= 0) {
			// 3 real roots
			double r     = sqrt(-p3);
			double theta = acos(-q/(2*r));
			double COS   = cos(theta/3);
			double SIN   = sqrt(3) * sin(theta/3);
			double CRr   = cbrt(r);
			x[0] =  CRr*COS*2     - M1;
			x[1] = -CRr*(COS+SIN) - M1;
			x[2] = -CRr*(COS-SIN) - M1;
			x[3] = 0;
		} else {	// D > 0
			// 1 real root and 2 complex roots
			double A = cbrt(0.5*(-q+sqrt(D)));
			double B = cbrt(0.5*(-q-sqrt(D)));
			x[0] = (A+B) - M1;
			x[1] = -0.5*(A+B) - M1;
			x[2] = x[1];
			x[3] = 0.5*(A-B)*sqrt(3);
		}
	}
	
	double solve_cubic(double a, double b, double c, double d)
	{
		double M1    = b/(3*a);
		double M2    = c/a;
		double M3    = d/a;
		double p     = M2/3 - sqr(M1);
		double q     = 2*cube(M1) - M1*M2 + M3;
		double r     = sqrt(-cube(p));
		double theta = acos(-q/(2*r));
		return 2*cbrt(r)*cos(theta/3) - M1;
	}
}

#endif
