// Setup class
// Kenji Sugase		12/21/2013

#include <iostream>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <cstring>
#include "Setup.h"

using namespace std;

Setup::Setup() : fVerb(0), prec(6), nLine(0), szToken(0), dimX(1), dimIndex(0) {}

Setup::~Setup() {}

// process command-line arguments
void Setup::ProcCmdLine(int argc, char *argv[], Xmgr &xmgr)
{
	if (argc == 1) {
		cerr << "ERROR: No parameter is specified.\n";
		ShowHelp(1, argv[0]);
	}
	for (int i = 1; i < argc; i++) {
		if      (!strcmp(argv[i], "-h"    )) { ShowHelp(0, argv[0]); }
		else if (!strcmp(argv[i], "-v"    )) { fVerb = 1;            }
		else if (!strcmp(argv[i], "-vv"   )) { fVerb = 2;            }
		else if (!strcmp(argv[i], "-noerr")) { xmgr.SetError(false); }
		else if (!strcmp(argv[i], "-xmgr" )) { xmgr.SetXmgr(true);   }
		else if (!strcmp(argv[i], "-i"    )) {
			if (++i >= argc) {
				cerr << "ERROR: No input file specified\n";
				exit(1);
			}
			inFile = argv[i];
		} else if (!strcmp(argv[i], "-o"  )) {
			if (++i >= argc) {
				cerr << "ERROR: No output file specified\n";
				exit(1);
			}
			outFile = argv[i];
		} else if (!strcmp(argv[i], "-d"  )) {
			inFile  = "glove.in";
			outFile = "glove.out";
		} else if (!strcmp(argv[i], "-dv") || !strcmp(argv[i], "-vd")) {
			fVerb = 1;
			inFile  = "glove.in";
			outFile = "glove.out";
		} else if (!strcmp(argv[i], "-dx") || !strcmp(argv[i], "-xd")) {
			xmgr.SetXmgr(true);
			inFile  = "glove.in";
			outFile = "glove.out";
		} else if (!strcmp(argv[i], "-dvx") || !strcmp(argv[i], "-dxv")
		        || !strcmp(argv[i], "-vdx") || !strcmp(argv[i], "-vxd")
		        || !strcmp(argv[i], "-xdv") || !strcmp(argv[i], "-xvd")) {
			fVerb = 1;
			xmgr.SetXmgr(true);
			inFile  = "glove.in";
			outFile = "glove.out";
		} else if (!strcmp(argv[i], "-vx") || !strcmp(argv[i], "-xv")) {
			fVerb = 1;
			xmgr.SetXmgr(true);
		} else if (!strcmp(argv[i], "-prec"  )) {
			if (++i >= argc) {
				cerr << "ERROR: No precision value specified\n";
				exit(1);
			}
			prec = strtol(argv[i], NULL, 0);
		} else {
			cerr << "ERROR: Parameter \'" << argv[i] << "\' not understood.\n";
			ShowHelp(1, argv[0]);
		}
	}
	if (outFile == "") {
		cerr << "ERROR: No output file is specified.\n";
		exit(1);
	}
}

// display the usage
void Setup::ShowHelp(int i, const char *exe)
{
	cerr << "GLOVE fitting program usage:\t" << exe << " {parameters}\n\n"
	     << "Valid parameters:\n"
	     << "  -i INPUT FILE\n"
	     << "  -o OUTPUT FILE\n"
	     << "  -d INPUT FILE = glove.in, OUTPUT FILE = glove.out\n"
	     << "  -v       Shows verbose messages\n"
     	     << "  -vv      Store the result of each method\n"
	     << "  -xmgr    Outputs Grace (XMGR) files\n"
	     << "  -noerr   Experimental error is set to 1.0\n"
	     << "  -h       Displays this help message\n";
	exit(i);
}

int Setup::IsVerb() const
{
	return fVerb;
}

int Setup::GetPrec() const
{
	return prec;
}

const string & Setup::GetOutputFile() const
{
	return outFile;
}

// read a GLOVE input file
void Setup::ReadInputFile(Funclib &func, Method &ctrl, ParDef &parm, Calc &calc, Xmgr &xmgr)
{
	// open the input file
	ifstream input(inFile.c_str());
	if (!input) {
		cerr << "ERROR: Unable to open \'" << inFile << "\'\n";
		exit(1);
	}

	// read the input file and look for tokens
	while (input) {
		++nLine;
		getline(input, textline);
		// check the linefeed code
		if (textline[textline.size()-1] == '\r') {
			textline.erase(textline.size()-1);
		}
		if (!TokenizeText(textline)) {}
		else if (IsMatch("FUNCTION")) { SetFUNCTION(func, parm, xmgr); }
		else if (IsMatch("METHOD")  ) { SetMETHOD(ctrl);               }
		else if (IsMatch("JACK1")   ) { SetJACK(calc, 1);              }
		else if (IsMatch("JACK")    ) { SetJACK(calc, 2);              }
		else if (IsMatch("MONTE1")  ) { SetMONTE(calc, 1, xmgr);       }
		else if (IsMatch("MONTE")   ) { SetMONTE(calc, 2, xmgr);       }
		else if (IsMatch("CALC")    ) { SetCALC(calc);                 }
		else if (IsMatch("PLOT")    ) { SetPLOT(xmgr);                 }
		else if (IsMatch("PLOT2")   ) { SetPLOT2(xmgr);                }
		else if (Is1stChar("@")     ) { SetXMGR(xmgr);                 }
		else if (IsMatch("GPAR")    ) { SetGPAR(parm);                 }
		else if (IsMatch("GREST")   ) { SetGREST(parm);                }
		else if (IsMatch("GFIXED")  ) { SetGFIXED(parm);               }
		else if (IsMatch("SET")     ) { SetSET(parm);                  }
		else if (IsMatch("#SET")    ) { SkipSET(parm);                 }
		else if (IsMatch("PAR")     ) { SetPAR(parm);                  }
		else if (IsMatch("REST")    ) { SetREST(parm);                 }
		else if (IsMatch("FIXED")   ) { SetFIXED(parm);                }
		else if (IsMatch("DATA")    ) { SetDATA(parm, xmgr);           }
		else if (Is1stChar("#")) {}
		else { ERROR_invalid_token(); }
	}
}

int Setup::TokenizeText(const string &textline)
{
	string delim = "\t, ";
	token.clear();

	// skip delim at beginning
	string::size_type lastPos = textline.find_first_not_of(delim, 0);
	// find first "non-delimiter"
	string::size_type pos     = textline.find_first_of(delim, lastPos);

	while (string::npos != pos || string::npos != lastPos){
	        // found a tokens, add it to the vector
	        token.push_back(textline.substr(lastPos, pos - lastPos));
	        // skip delim
	        lastPos = textline.find_first_not_of(delim, pos);
	        // find next "non-delimiter"
	        pos = textline.find_first_of(delim, lastPos);
	}
	return szToken = token.size();
}

void Setup::SetFUNCTION(Funclib &func, ParDef &parm, Xmgr &xmgr)
{
	// check the number of parameters
	if (szToken != 2) {
		ERROR_wrong_parm_num("FUNCTION");
	}

	func.Pick_Func(token[1]);
	fFUNC = true;
	dimX = func.Get_dimX();
	dimIndex = func.Get_dimIndex();
	parm.setRule(func.Get_Rules());
	xmgr.header(func.XmgrHeader());		// XMGR default header
}

void Setup::SetMETHOD(Method &ctrl)
{
	// check the number of parameters
	if (szToken < 2 || szToken > 6) {
		ERROR_wrong_parm_num("METHOD");
	}
	ctrl.Add(token);
}

void Setup::resetMETHOD_ONEEX(Method &ctrl)
{
	ctrl.Clear();
	textline = "METHOD INIT";
	TokenizeText(textline);
	ctrl.Add(token);
	textline = "METHOD ONEEX";
	TokenizeText(textline);
	ctrl.Add(token);
}


void Setup::SetJACK(Calc &calc, int type)
{
	// check the number of parameters
	if (szToken != 1) {
		ERROR_wrong_parm_num("JACK");
	}

	calc.SetJack(type);
}

void Setup::SetMONTE(Calc &calc, int num, const Xmgr &xmgr)
{
	// check the number of parameters
	if (szToken != 2) {
		ERROR_wrong_parm_num("MONTE");
	}
	// Monte Carlo simulations cannot be run without experimental error.
	if (!xmgr.isError()) {
		ERROR_no_MONTE_without_error();
	}
	int iter = GetInt(1);
	if (iter < 2) {
		ERROR_too_few_iter();
	}
	calc.SetMonte(num, iter);
}

void Setup::SetCALC(Calc &calc)
{
	// Is FUNCTION specified?
	IsFUNCTION();
	// check the number of parameters
	if (szToken != dimX+4) {
		ERROR_wrong_parm_num("CALC");
	}

	string type     = token[1];			// calculation type
	string label    = token[2];			// label for calculation
	dvect condition = GetDataX(3);			// experimental conditions
	ivect index     = GetIndex(token[dimX+3]);	// index for experimental conditions
	calc.Add(type, label, condition, index);
}

void Setup::SetPLOT(Xmgr &xmgr)
{
	// Is FUNCTION specified?
	IsFUNCTION();
	// check the number of parameters
	if (szToken < 2 || szToken > dimX+6) {
		ERROR_wrong_parm_num("PLOT");
	}

	string label = token[1];			// label for plot
	dvect condition;				// experimental conditions
	ivect index;					// index for experimental condition
	if (szToken < dimX+3) {
		condition.push_back(0.0);
		index.push_back(0);
	} else {
		condition = GetDataX(2);
		if (dimIndex) {
			index = GetIndex(token[dimX+2]);
		} else {
			index.push_back(0);
		}
	}

	if (szToken < dimX+6) {
		xmgr.add_plot(label, condition, index);
	} else {
		double start = GetDouble(dimX+3);	// plot start value
		double end   = GetDouble(dimX+4);	// plot end value
		if (start > end) { ERROR_wrong_range("PLOT"); }
		int size  = GetInt(dimX+5);		// plot size
		xmgr.add_plot(label, condition, index, start, end, size);
	}
}

void Setup::SetPLOT2(Xmgr &xmgr)
{
	// Is FUNCTION specified?
	IsFUNCTION();
	// check the number of parameters
	if (szToken < 2 || szToken > dimX+7) {
		ERROR_wrong_parm_num("PLOT2");
	}

	string label = token[1];		// label for plot
	dvect condition;			// experimental conditions
	ivect index;				// index for experimental condition
	condition = GetDataX(2);
	index = GetIndex(token[dimX+2]);
	double option = GetDouble(dimX+3);
	double start  = GetDouble(dimX+4);	// plot start value
	double end    = GetDouble(dimX+5);	// plot end value
	int size      = GetInt(dimX+6);		// plot size
	xmgr.add_plot(label, condition, index, start, end, size, option);
}

void Setup::SetXMGR(Xmgr &xmgr)
{
	xmgr.add_XMGRcommand(textline.c_str());
}

void Setup::SetGPAR(ParDef &parm)
{
	// Is FUNCTION specified?
	IsFUNCTION();
	// check the number of parameters
	if (szToken < 3 || szToken > 6) {
		ERROR_wrong_parm_num("GPAR");
	}

	string name  = token[1];			// parameter name
	double start = GetDouble(2);			// parameter start value
	double end   = GetDouble(3);			// parameter end value
	if (start > end) { ERROR_wrong_range("GPAR"); }
	int size     = 1;				// grid size
	int interval = GRID_NORMAL;
	if (szToken >= 5) {
		if (IsLastChar(4, "e")) {
			interval = GRID_EXP;
		} else if (IsLastChar(4, "g")) {
			interval = GRID_GAUSS;
		} else if (IsLastChar(4, "t")) {
			interval = GRID_TENTH;
		} else if (IsLastChar(4, "E")) {
			interval = GRID_iEXP;
		} else if (IsLastChar(4, "G")) {
			interval = GRID_iGAUSS;
		} else if (IsLastChar(4, "T")) {
			interval = GRID_iTENTH;
		}
		size = GetInt(4);
	}
	double initial = HUGE_VAL;
	if (szToken == 6) { initial = GetDouble(5); }
	parm.Add_GPAR(name, start, end, size, interval, initial);
}

void Setup::SetGREST(ParDef &parm)
{
	// Is FUNCTION specified?
	IsFUNCTION();
	// check the number of parameters
	if (szToken < 4 || szToken > 6) {
		ERROR_wrong_parm_num("GREST");
	}

	string name  = token[1];			// parameter name
	double start = GetDouble(2);			// parameter start value
	double end   = GetDouble(3);			// parameter end value
	if (start > end) { ERROR_wrong_range("GREST"); }
	int size     = 1;				// grid size
	int interval = GRID_NORMAL;
	if (szToken >= 5) {
		if (IsLastChar(4, "e")) {
			interval = GRID_EXP;
		} else if (IsLastChar(4, "g")) {
			interval = GRID_GAUSS;
		} else if (IsLastChar(4, "t")) {
			interval = GRID_TENTH;
		} else if (IsLastChar(4, "E")) {
			interval = GRID_iEXP;
		} else if (IsLastChar(4, "G")) {
			interval = GRID_iGAUSS;
		} else if (IsLastChar(4, "T")) {
			interval = GRID_iTENTH;
		}
		size = GetInt(4);
	}
	double initial = HUGE_VAL;
	if (szToken == 6) { initial = GetDouble(5); }
	parm.Add_GREST(name, start, end, size, interval, initial);
}

void Setup::SetGFIXED(ParDef &parm)
{
	// Is FUNCTION specified?
	IsFUNCTION();
	// check the number of parameters
	if (szToken != 3) {
		ERROR_wrong_parm_num("GFIXED");
	}

	string name  = token[1];			// parameter name
	double value = GetDouble(2);			// fixed parameter value
	parm.Add_GFIXED(name, value);
}

void Setup::SetSET(ParDef &parm)
{
	// Is FUNCTION specified?
	IsFUNCTION();
	// check the number of parameters
	if (szToken != 2) {
		ERROR_wrong_parm_num("SET");
	}
	fSET = true;
	fSkipSET = false;
	parm.Add_Set(token[1]);
}

void Setup::SkipSET(ParDef &parm)
{
	// Is FUNCTION specified?
	IsFUNCTION();
	// check the number of parameters
	if (szToken != 2) {
		ERROR_wrong_parm_num("SET");
	}
	fSkipSET = true;
}

void Setup::SetPAR(ParDef &parm)
{
	// The SET is skipped
	if (fSkipSET) { return; }
	// Is FUNCTION specified?
	IsFUNCTION();
	// no dataset is defined
	IsSET("PAR");
	// check the number of parameters
	if (szToken < 3 || szToken > 6) {
		ERROR_wrong_parm_num("PAR");
	}

	string name = token[1];				// parameter name
	double start, end;
	int interval = GRID_NORMAL;
	if (szToken == 3) {
		if (IsNumeric(token[2])) {
			// floating local parameter
			start = strtod(token[2].c_str(), NULL);
			parm.Add_PAR(name, start, start, 1, interval, start);
		} else {
			// map the local parameter to a global parameter
			string GPAR = token[2];
			parm.Add_PAR(name, GPAR);
		}
	} else {
		start = GetDouble(2);			// parameter start value
		end   = GetDouble(3);			// parameter end value
		if (start > end) { ERROR_wrong_range("PAR"); }
		int size = 1;				// grid size
		if (start < end)  { size = 2; }
		if (szToken >= 5) {
			if (IsLastChar(4, "e")) {
				interval = GRID_EXP;
			} else if (IsLastChar(4, "g")) {
				interval = GRID_GAUSS;
			} else if (IsLastChar(4, "t")) {
				interval = GRID_TENTH;
			} else if (IsLastChar(4, "E")) {
				interval = GRID_iEXP;
			} else if (IsLastChar(4, "G")) {
				interval = GRID_iGAUSS;
			} else if (IsLastChar(4, "T")) {
				interval = GRID_iTENTH;
			}
			size = GetInt(4);
		}
		double initial = HUGE_VAL;
		if (szToken == 6) { initial = GetDouble(5); }
		parm.Add_PAR(name, start, end, size, interval, initial);
	}
}

void Setup::SetREST(ParDef &parm)
{
	// The SET is skipped
	if (fSkipSET) { return; }
	// Is FUNCTION specified?
	IsFUNCTION();
	// no dataset is defined
	IsSET("REST");
	// check the number of parameters
	if (szToken < 4 || szToken > 6) { ERROR_wrong_parm_num("REST"); }

	string name  = token[1];			// parameter name
	double start = GetDouble(2);			// parameter start
	double end   = GetDouble(3);			// parameter end
	if (start > end) { ERROR_wrong_range("REST"); }
	int size     = 1;				// grid size
	int interval = GRID_NORMAL;
	if (szToken >= 5) {
		if (IsLastChar(4, "e")) {
			interval = GRID_EXP;
		} else if (IsLastChar(4, "g")) {
			interval = GRID_GAUSS;
		} else if (IsLastChar(4, "t")) {
			interval = GRID_TENTH;
		} else if (IsLastChar(4, "E")) {
			interval = GRID_iEXP;
		} else if (IsLastChar(4, "G")) {
			interval = GRID_iGAUSS;
		} else if (IsLastChar(4, "T")) {
			interval = GRID_iTENTH;
		}
		size = GetInt(4);
	}
	double initial = HUGE_VAL;
	if (szToken == 6) { initial = GetDouble(5); }
	parm.Add_REST(name, start, end, size, interval, initial);
}

void Setup::SetFIXED(ParDef &parm)
{
	// The SET is skipped
	if (fSkipSET) { return; }
	// Is FUNCTION specified?
	IsFUNCTION();
	// no dataset is defined
	IsSET("FIXED");
	// check the number of parameters
	if (szToken != 3) {
		ERROR_wrong_parm_num("FIXED");
	}

	string name  = token[1];			// Parameter name
	double value = GetDouble(2);			// fixed parameter value
	parm.Add_FIXED(name, value);
}

void Setup::SetDATA(ParDef &parm, Xmgr &xmgr)
{
	// SET is skipped
	if (fSkipSET) { return; }
	// Is FUNCTION specified?
	IsFUNCTION();
	// no dataset is defined
	IsSET("DATA");
	// check the number of parameters
	if (xmgr.isError()) {
		if (szToken - dimX != 4) {
			if (szToken - dimX == 3) {
				cerr << "WARNING: Assuming that no experimental error is specficed." << endl;
				xmgr.SetError(false);
			} else {
				ERROR_wrong_parm_num("DATA");
			}
		}
	} else {
		if (szToken - dimX != 3 && szToken - dimX != 4 ) {
			ERROR_wrong_parm_num("DATA");
		}
	}
	dvect condition = GetDataX(1);		// experimental conditions
	ivect index;				// index for experimental condition
	double value;				// dependent experimental value
	double error = 1.0;			// experimental error
	if (dimIndex == 0) {
		index.push_back(0);
		value = GetDouble(dimX+1);
		if (xmgr.isError()) { error = GetDouble(dimX+2); }
	} else {
		index = GetIndex(token[dimX+1]);
		value = GetDouble(dimX+2);
		if (xmgr.isError()) { error = GetDouble(dimX+3); }
	}
	parm.Add_DATA(condition, index, value, error);
}

bool Setup::Is1stChar(const char *letter)
{
	return token[0].c_str()[0] == *letter;
}

bool Setup::IsLastChar(size_t pos, const char *letter)
{
	int size = token[pos].size();
	return token[pos].c_str()[size-1] == *letter;
}

bool Setup::IsMatch(const char *word)
{
	return !strcasecmp(token[0].c_str(), word);
}

bool Setup::IsNumeric(const string &s)
{
	stringstream ss(s);
	double d;
	ss >> d;
	if (ss.fail()) {
		return false;
	}
	return true;
}

double Setup::GetDouble(size_t pos)
{
	return strtod(token[pos].c_str(), NULL);
}

int Setup::GetInt(size_t pos)
{
	return strtol(token[pos].c_str(), NULL, 0);
}

dvect Setup::GetDataX(size_t pos)
{
	dvect datax;
	for (int i = 0; i < dimX; i++) {
		if (token[pos+i] == "*") {
			datax.push_back(0.0);	// dummy x value (= 0.0)
		} else {
			datax.push_back(GetDouble(pos+i));
		}
	}
	return datax;
}

ivect Setup::GetIndex(const string &str)
{
	ivect index;

	if (static_cast<int>(str.length()) != dimIndex) {
		cerr << "ERROR: Index is not defined correctly.\n";
		ShowOffendingLine();
	}
 	for (int i = 0; i < dimIndex; i++) {
		index.push_back(strtol(str.substr(i, 1).c_str(), NULL, 0));
	}
	return index;
}

void Setup::IsFUNCTION()
{
	if (!fFUNC) { ERROR_no_FUNCTION(); }
}

void Setup::IsSET(const char *word)
{
	if(!fSET) {ERROR_no_SET(word); }
}
void Setup::ERROR_invalid_token()
{
	cerr << "ERROR: Keyword \'" << token[0] << "\' not recognized.\n";
	ShowOffendingLine();
}

void Setup::ERROR_no_FUNCTION()
{
	cerr << "ERROR: No \'FUNCTION\' specified.\n";
}

void Setup::ERROR_no_MONTE_without_error()
{
	cerr << "ERROR: Unable to perform Monte Carlo error analysis without a measure of\n"
	     << " the uncertainty of each point.\n";
	exit(1);
}

void Setup::ERROR_too_few_iter()
{
	cerr << "ERROR: Monte Carlo simulations require more than 2 iterations.\n";
	exit(1);
}

void Setup::ERROR_no_SET(const char *token)
{
	cerr << "ERROR: Keyword \'" << token << "\' cannot preceed the \'SET\' keyword.\n";
	ShowOffendingLine();
}

void Setup::ERROR_wrong_parm_num(const char *token)
{
	cerr << "ERROR: The number of parameters for \'" << token << "\' is wrong.\n";
	ShowOffendingLine();
}

void Setup::ERROR_wrong_range(const char *token)
{
	cerr << "ERROR: Keyword \'" << token << "\' has a wrong parameter range.\n";
	ShowOffendingLine();
}

void Setup::ShowOffendingLine()
{
	cerr << "Line " << nLine << ": " << textline << "\n";
	exit(1);
}
