// Setup class
// Kenji Sugase		11/14/2012

#ifndef __SETUP_H__
#define __SETUP_H__

#include "Type.h"
#include "FuncLib.h"
#include "ParDef.h"
#include "Method.h"
#include "Calc.h"
#include "Xmgr.h"

class Setup {

private:
	int fVerb;		// 0: non verbose  1: verbose  2: very verbose
	int prec;		// output precision
	bool fFUNC;		// whether function defined?
	bool fSET;		// whether SET defined?
	bool fSkipSET;		// whether SET comment out?
	string inFile;		// glove input file
	string outFile;		// glove output file
	string textline;	// textline in the input file
	int nLine;		// line number
	int szToken;		// the number of words in each line
	int dimX;		// the number of data dimension
	int dimIndex;		// the number of index dimension
	svect token;		// split words

	void ShowHelp(int i, const char *exe);
	int TokenizeText(const string &textline);
	void SetFUNCTION(Funclib &func, ParDef &parm, Xmgr &xmgr);
	void SetMETHOD(Method &ctrl);
	void SetMONTE(Calc &calc, int num, const Xmgr &xmgr);
	void SetJACK(Calc &calc, int type);
	void SetCALC(Calc &calc);
	void SetPLOT(Xmgr &xmgr);
	void SetPLOT2(Xmgr &xmgr);
	void SetXMGR(Xmgr &xmgr);
	void SetGPAR(ParDef &parm);
	void SetGREST(ParDef &parm);
	void SetGFIXED(ParDef &parm);
	void SetSET(ParDef &parm);
	void SkipSET(ParDef &parm);
	void SetPAR(ParDef &parm);
	void SetREST(ParDef &parm);
	void SetFIXED(ParDef &parm);
	void SetDATA(ParDef &parm, Xmgr &xmgr);
	bool Is1stChar(const char *letter);
	bool IsMatch(const char *token);
	bool IsLastChar(size_t pos, const char *letter);
	bool IsNumeric(const string &s);
	double GetDouble(size_t pos);
	int GetInt(size_t pos);
	dvect GetDataX(size_t pos);
	ivect GetIndex(const string &str);
	void IsFUNCTION();
	void IsSET(const char *word);
	void ERROR_invalid_token();
	void ERROR_no_FUNCTION();
	void ERROR_no_MONTE_without_error();
	void ERROR_too_few_iter();
	void ERROR_no_SET(const char *token);
	void ERROR_wrong_parm_num(const char *token);
	void ERROR_wrong_range(const char *token);
	void ShowOffendingLine();

	Setup(const Setup &);			// forbidden
	Setup & operator=(const Setup &);	// forbidden

public:
	Setup();
	~Setup();

	void ProcCmdLine(int argc, char *argv[], Xmgr &xmgr);
	void ReadInputFile(Funclib &func, Method &ctrl, ParDef &parm, Calc &calc, Xmgr &xmgr);
	void resetMETHOD_ONEEX(Method &ctrl);
	const string & GetOutputFile() const;
	int IsVerb() const;
	int GetPrec() const;
};

#endif
