// Alloc class		Allocation and deallocation routines
// Jonathan C. Lansing	6/25/04
// Kenji Sugase		10/2/09

#ifndef __ALLOC_H__
#define __ALLOC_H__

#include <iostream>
#include <cstdlib>

using namespace std;
namespace {
	void err_alloc(const char *routine) {
		cerr << "ERROR: Unable to allocate memory in routine " << routine << endl;
		exit(1);
	}
}

template <class T> int make_new(T *& ptr, const char* routine) {
	try {
		ptr = new T;
		if (ptr == NULL) { throw ptr; }
	} catch (...) {
		err_alloc(routine);
	}
	return 0;
}

template <class T> int make_new(T *& ptr, int sz, const char *routine) {
	try {
		ptr = new T [sz];
		if (ptr == NULL) { throw ptr; }
	} catch (...) {
		err_alloc(routine);
	}
	return 0;
}

template <class T> int make_new(T **& ptr, int sz1, int sz2, const char *routine) {
	try {
		ptr = new T * [sz1];
		if (ptr == NULL) { throw ptr; }
	} catch (...) {
		err_alloc(routine);
	}
	for (int i = 0; i < sz1; i++) {
		try {
			if (ptr == NULL) { throw ptr; }
			ptr[i] = new T [sz2];
		} catch (...) {
			err_alloc(routine);
		}
	}
	return 0;
}

// Delete allocated memory for a 1D array
template <class T> void zap(T &ptr) {
	delete [] ptr;
	ptr = NULL;
}

// Delete allocated memory for a 2D array
template <class T> void zap(T &ptr, int sz) {
	for (int i = 0; i < sz; i++) { zap(ptr[i]); }
	delete [] ptr;
	ptr = NULL;
}

// Delete allocated memory for a 3D array
template <class T> void zap(T &ptr, int sz1, int sz2) {
	for (int i = 0; i < sz1; i++) {	zap(ptr[i], sz2); }
	delete [] ptr;
	ptr = NULL;
}

#endif
