// func_cleanex.cpp
// Kenji Sugase		11/12/2012

#include <iostream>
#include <cmath>
#include <cstdlib>
#include "Data.h"
#include "FuncLib.h"
#include "Type.h"
#include "Mathlib.h"

using namespace std;

// CLEANEX-PM: f(tm) = k / (R1a - R1b + k) * {-exp(-R1a - k)tm + exp(-R1btm)}
// par[0]	k	(s-1)
// par[1]	R1a	(s-1)
// par[2]	R1b	(s-1)
double fcleanex(const Data &data, double *par, dvect &dy, const bvect &fixed)
{
	// define index
	const int i_k   = data.map[0];
	const int i_R1a = data.map[1];
	const int i_R1b = data.map[2];
	// copy parameters
	double tm  = data.x[0];
	double k   = par[0];
	double R1a = par[1];
	double R1b = par[2];
	// calculate parameters
	double denom = 1 / (R1a - R1b + k + TINY);
	if ( R1b*tm > EXP_MAX || (R1a + k)*tm > EXP_MAX ){ return HUGE_VAL; }
	double expR1a = exp((-R1a - k) * tm);
	double expR1b = exp(-R1b * tm);
	if (dy.size()) {
		double denom2 = sqr(denom);
		dy[i_k]   = (denom - denom2 * k) * (-expR1a + expR1b) + denom * k * tm * expR1a;
		dy[i_R1a] = -denom2 * k * (-expR1a + expR1b) + denom * k * tm * expR1a;
		dy[i_R1b] =  denom2 * k * (-expR1a + expR1b) - denom * k * tm * expR1b;
	}
	return denom * k * (-expR1a + expR1b);
}

int Funclib::Read_CLEANEX_Funcs(int idx)
{
	// CLEANEX-PM
	idx = Define_Func(&fcleanex, "CLEANEX"
		, "f(k, R1a, R1b)", "CLEANEX", XDIM2, IDIM1);
	Define_Parm(idx, "k"    ,     0,  2500, "0");
	Define_Parm(idx, "R1a"  ,     0,   100, "1");
	Define_Parm(idx, "R1b"  ,     0,   100, "1");
	
	return idx;
}
