// Funclib class
//	Library of functions for use in data fitting
// Kenji Sugase			11/25/2013

#include <cstring>
#include <cstdlib>
#include "FuncLib.h"

using namespace std;

Funclib::Funclib() : num(-1) {
	int idx = 0;
	Read_Exp_Funcs(idx);
	Read_Bio_Funcs(idx);
	Read_CPMG2_Funcs(idx);
	Read_CPMG3_Funcs(idx);
	Read_MQCPMG_Funcs(idx);
	Read_R1RHO_Funcs(idx);
	Read_CLEANEX_Funcs(idx);
}
Funclib::~Funclib() {};


// Define a function
int Funclib::Define_Func(pFunc pfx, const char *name, const char *eqn, const char *xmgr, int xdim, int idim, int dist)
{
	pFuncs.push_back(pfx);
	func_names.push_back(name);
	func_eqns.push_back(eqn);
	xmgr_headers.push_back(xmgr);
	func_xdims.push_back(xdim);
	func_idims.push_back(idim);
	func_rnd_dists.push_back(dist);
	if (dist == LOG) {
		func_rnd_tau.push_back(1); 	// data.x[1] is a Tcpmg value
	} else {
		func_rnd_tau.push_back(0);	// Not used
	}

	return pFuncs.size()-1;
}

// Define a parameter
void Funclib::Define_Parm(int num, const char *name, double min, double max, const char *index)
{
	int sz = pFuncs.size();
	func_rules.resize(sz);
	func_rules[num].Define_Parm(name, min, max, index);
}

// Select a function from the library
void Funclib::Pick_Func(const string &name)
{
	int sz = pFuncs.size();
	for (int i = 0; i < sz; i++) {
		if (!strcasecmp(func_names[i].c_str(), name.c_str())) {
			num =  i;
			fx = pFuncs[i];
			return;
		}
	}
	ERROR_invalid_func(name);
}

void Funclib::ERROR_invalid_func(const string &name)
{
	cerr << "ERROR: Function name \'" << name << "\' is not recognized.\n"
	     << "Valid function names: ";
	// show all valid function names
	int sz = pFuncs.size();
	for (int i = 0; i < sz; i++) {
		cerr << "\'" << func_names[i] << "\' ";
	}
	cerr << endl;
	exit(1);
}
const Rules & Funclib::Get_Rules() const
{
	return func_rules[num];
}

const string & Funclib::Name() const
{
	return func_names[num];
}

const string & Funclib::Eqn() const
{
	return func_eqns[num];
}

const string & Funclib::XmgrHeader() const
{
	return xmgr_headers[num];
}

int Funclib::Get_dimX() const
{
	return func_xdims[num];
}
int Funclib::Get_dimIndex() const
{
	return func_idims[num];
}

int Funclib::Get_dist() const
{
	return func_rnd_dists[num];
}
int Funclib::posTau() const
{
	return func_rnd_tau[num];
}
