// Prototypes for linear algebra routines from BLAS and LAPACK
// Prototypes for wrapper routines also provided
// All matrices are for double precision complex matrices

#include <iostream>
#include <vector>
#include "Type.h"

using namespace std;

// Extern "C" statement needed to prevent C++ decoration of function names
extern "C" {
	// BLAS routine to copy all/part of a matrix
	void zlacpy_(const char *UPLO, int *M, int *N, const double *A, int *LDA, double *B, int *LDB);

	// Matrix-matrix multiplication (BLAS Level 3)
	void zgemm_(const char *TRANSA, const char *TRANSB, int *M, int *N, int *K, double *ALPHA
		, const double *A, int *LDA, const double *B, int *LDB, double *BETA, double *C, int *LDC);

	// Matrix-vector multiplication (BLAS level 2)
	void zgemv_(const char *TRANS, int *M, int *N, double *ALPHA, const double *A, int *LDA
		, const double *X, int *INCX, double *BETA, double *Y, int *INCY);

	// LU decomposition
	void zgetrf_(int *M, int *N, double *A, int *LDA, int *IPIV, int *INFO);

	// Matrix inversion following LU decomposition (zgetrf)
	void zgetri_(int *N, double *A, int *LDA, int *IPIV, double *WORK, int *LWORK, int *INFO);

	// Determine eigenvalues and eigenvectors
	void zgeev_(const char *JOBVL, const char *JOBVR, int *N, double *A, int *LDA, double *W
		, double *VL , int *LDVL, double *VR, int *LDVR, double *WORK, int *LWORK
		, double *RWORK, int *INFO);

	// Solve a normal equation using Cholesky decomposition
//	void dppsv_(const char *UPLO, int *N, int *NRHS, double *A, double *B, int *LDB, int *INFO);

	// LU decomposition
	void dgetrf_(int *M, int *N, double *A, int *LDA, int *IPIV, int *INFO);

	// Solve a normal equation using LU decomposition
	void dgesv_(int *N, int *NRHS, double *A, int *LDA, int *IPIV, double *B, int *LDB, int *INFO);

	// Matrix inversion following LU decomposition (dgetrf)
	void dgetri_(int *N, double *A, int *LDA, int *IPIV, double *WORK, int *LWORK, int *INFO);

}

int idxr(int c, int r, int order);
int idxi(int c, int r, int order);
int idvr(int c);
int idvi(int c);

// B = A
void zmat_cpy(int order, const double *A, double *B);

// C = A x B
void zmat_mlt(int order, const double *A, const double *B, double *C);

// D = A x B x C
void zmat_mlt(int order, const double *A, const double *B, const double *C, double *D);

// E = A x B x C x D
void zmat_mlt(int order, const double *A, const double *B, const double *C, const double *D, double *E);

// Y = A * X for matrix A and vector X
void zmat_vmlt(int order, const double *A, const double *X, double *Y);

// Y = A * B * X for matrices A, B and vector X
void zmat_vmlt(int order, const double *A, const double *B, const double *X, double *Y);

// C = A x B (A is a diagonal matrix)
void zdmat_mlt(int order, const double *A, const double *B, double *C);

// D = A x B x C (A is a diagonal matrix)
void zdmat_mlt(int order, const double *A, const double *B, const double *C, double *D);

// exp(A*x)
void zmat_exp(int order, double x, const double *A, double *B);

// A*
void zmat_cnj(int order, const double *A, double *cnjA);

// A-1
int zmat_inv(int order, const double *mat, double *imat);

// Compute right eigenvectors and eigenvalues
int zmat_eig(int order, double *mat, double *evec, double *eval);
int zmat_eig(int order, const double *mat, double *evec, double *ievec, double *eval);

void zmat_pow(int order, double *A, int n, double *B);
void zmat_mltpow(int order, const double *A, const double *B, const double *C, const double *D, int n, double *E);
//void zmat_powfast(int order, double *A, int n, double *B);
//void zmat_mltpowfast(int order, const double *A, const double *B, const double *C, const double *D, int n, double *E);

// Solve a normal equation: X = (A-1) x B
int LM_solv(int order, const d2vect &A, const dvect &B, dvect &X, double lambda);

// Compute an inverse matrix: A-1
int LM_inv(int order, d2vect &mat);
