// Rules class
//	Object to handle the rules for marking up parameter names
// Jonathan C. Lansing	3/6/05
// Kenji Sugase		10/8/09

#include <iostream>
#include <cstring>
#include <cstdlib>
#include "Rule.h"

using namespace std;

Rules::Rules() {}
Rules::~Rules() {}

void Rules::Define_Parm(const char *name, double min, double max, const char *index)
{
	parm_name.push_back(name);
	parm_min.push_back(min);
	parm_max.push_back(max);
	parm_index.push_back(index);
}

void Rules::markup(string &str, int i, const ivect &idx)
{
	str = parm_name[i];
	int j;
	string num;
	if ((j = index(str)) < 0) {
		cerr << "ERROR: Markup rule not found for parameter \'" << str << "\'\n";
		exit(1);
	}
	if (&idx != NULL && parm_index[j].size() > idx.size()) {
		cerr << "ERROR (Rules::markup): Index list too short\n";
		exit(1);
	}
	int sz = parm_index[i].size();
	for (int k = 0; k < sz; k++) {
		if (parm_index[j].c_str()[k] != '0') {
			if (&idx == NULL) {
				num = "0";
			} else {
				num = "";
				append_int(num, idx[k]);
			}
			str += "_" + num;
		}
	}
}

// Convert an integer into characters and append it to a string
void Rules::append_int(string &str, int i)
{
	if (i > 9) append_int(str, i/10);
	str += char(i % 10 + 48);
}

// Return the base paramter number
int Rules::index(const string &str)
{
	// Search the base parameter name in those defined for the model
	int nb, sz = parm_name.size();
	for (nb = 0; nb < sz; nb++) {
		if (!strcasecmp(parm_name[nb].c_str(), str.c_str())) {
			break;
		}
	}

	if (nb < sz ) { return nb;  }
	else          { return -1;  }
}

// Set index for each paramter and return the base paramter number
int Rules::decompose(const string &PAR, ivect &v)
{
	string bpar;		// base parameter name
	string indxes;		// indexes of the parameter
	int i;
	// split the string on underscore marks
	if ((i = PAR.find("_")) == (int)string::npos) {
		bpar   = PAR;
		indxes = "";
	} else {
		// PAR includes "_" but dosen't index number
		int sz = PAR.size();
		if (i + 1 >= sz) {
			cerr << "ERROR: Badly constructed parameter \'" << PAR << "\'\n";
			exit(1);
		}
		// get the base parameter name
		bpar.assign(PAR, 0, i);

		// get index, e.g. 0_0_0
		indxes.assign(PAR, i+1, PAR.size());
	}

	int nb;			// base parameter number
	// match the base parameter name
	if ((nb = index(bpar)) == -1) {
		cerr << "ERROR: Parameter name \'" << PAR << "\' unrecognized.\n";
		cerr << "Valid parameter names for the model:";
		int sz = parm_name.size();
		for (int i = 0; i < sz; i++) { cerr << "\t\'" << parm_name[i] << "\'"; }
		cerr << "\n";
		exit(1);
	}

	// If it's all undefined, then assume _0_0...
	string idx1;		// split index of the parameter
	int j;
	int sz = parm_index[nb].size();
	if (indxes == "") {
		for (int i = 0; i < sz; i++) { v.push_back(-1); }
	} else {
		for (int i = 0; i < sz; i++) {
			if (parm_index[nb].c_str()[i] != '0') {
				if (indxes.size() == 0) {
					cerr << "ERROR: Insufficient indices on par \'"
					     << PAR << "\'\n";
					exit(1);
				}
				if ((j = indxes.find("_")) == (int)string::npos) {
					idx1  = indxes;
					indxes = "";
				} else {
					int isz = indxes.size();
					if (j + 1 >= isz) {
						cerr << "ERROR: Badly constructed parameter \'"
						     << PAR << "\'\n";
						exit(1);
					}
					idx1.assign(indxes, 0, j);
					indxes.assign(indxes, j + 1, indxes.size());
				}
				v.push_back(strtol(idx1.c_str(), NULL, 0));
			} else {
				v.push_back(-1);
			}
		}
	}
	if (indxes.size()) {
		cerr << "ERROR: Parameter name \'" << PAR << "\' has too many " << "indices\n";
		exit(1);
	}
	return nb;
}

// Get the number of base parameters
int Rules::basePAR_size() const
{
	return parm_name.size();
}

double Rules::Get_LowerBound(size_t num) const
{
	return parm_min[num];
}

double Rules::Get_UpperBound(size_t num) const
{
	return parm_max[num];
}
