// GLOVE - Global and Local Optimization of Variable Expressions
// Kenji Sugase, Tsuyoshi Konuma, Jonathan C. Lansing, Peter E. Wright
// The Scripps Research Institute
// Suntory Foundation for Life Sciences
//
// GLOVE fits data to a function using a variety of minimization algorithms.
// Each parameter may be optimized locally or globally.

#include "Setup.h"
#include "FuncLib.h"
#include "ParDef.h"
#include "Method.h"
#include "Xmgr.h"
#include "Fit.h"

using namespace std;

int main(int argc, char *argv[])
{
	const string VERSION = "2.4.4";

	// process command line
	Setup setup;
	Xmgr xmgr;
	setup.ProcCmdLine(argc, argv, xmgr);

	// read input file
	Funclib func;
	ParDef parm;
	Method method, method_old;
	Calc calc;
	setup.ReadInputFile(func, method, parm, calc, xmgr);
	parm.prep();

	// start fitting
	Fit fit(VERSION, func, method, parm, calc, xmgr, setup.GetOutputFile(), setup.IsVerb(), setup.GetPrec());
	fit.run_FIT();

	// report the fitting results
	fit.report_fit(method.size());

	// plot graphs
	if (xmgr.isXmgr()) { fit.plot_xmgr(); }

	// error estimation by Jackknife simulation
	int nsim = 0;
	if (calc.getJackType() == 1) {
		method_old = method;
		setup.resetMETHOD_ONEEX(method);
		fit.run_JACK();
		fit.report_final(SIM_JACK);
		if (xmgr.isXmgr()) { fit.plot_xmgr(++nsim, "Jackknife"); }
		method = method_old;
	} else if (calc.getJackType() == 2) {
		fit.run_JACK();
		fit.report_final(SIM_JACK);
		if (xmgr.isXmgr()) { fit.plot_xmgr(++nsim, "Jackknife"); }
	}

	// error estimation by Monte carlo simulation
	if (calc.getMonteType() == 1) {
		setup.resetMETHOD_ONEEX(method);
		fit.run_MONTE();
		fit.report_final(SIM_MONTE);
		if (xmgr.isXmgr()) { fit.plot_xmgr(++nsim, "Monte Carlo"); }
	} else if (calc.getMonteType() == 2) {
		fit.run_MONTE();
		fit.report_final(SIM_MONTE);
		if (xmgr.isXmgr()) { fit.plot_xmgr(++nsim, "Monte Carlo"); }
	}
	return 0;
}
