// Method class
// Kenji Sugase		11/25/2013

#include <iostream>
#include <iomanip>
#include <sstream>
#include <cstring>
#include <cstdlib>
#include "Method.h"

using namespace std;

Method::Method()
{
	svect words;
	words.push_back("METHOD");
	words.push_back("INIT");
	Add(words);
}

Method::~Method(){}

void Method::Clear()
{
	ctrl.resize(0);
}

void Method::Add(const svect &words)
{
	OneMethod method;

	int sz = words.size();
	if (isMatch(words[1], "INIT")) {
		method.type = METHOD_INIT;
		ctrl.push_back(method);
	} else if (isMatch(words[1], "GRID")) {
		method.type = METHOD_GRID;
		ctrl.push_back(method);
	} else if (isMatch(words[1], "ONE")) {
		method.type = METHOD_ONE;
		ctrl.push_back(method);
	} else if (isMatch(words[1], "ONEEX")) {
		method.type = METHOD_ONEEX;
		ctrl.push_back(method);
	} else if (isMatch(words[1], "RANDOM")) {
		method.type = METHOD_RANDOM;
		check_option(sz, 3, "RANDOM");
		method.iter = str2long(words[2]);
		ctrl.push_back(method);
	} else if (isMatch(words[1], "MCMIN")) {
		method.type = METHOD_MCMIN;
		check_option(sz, 4, "MCMIN");
		method.iter  = str2long(words[2]);
		method.scale = str2double(words[3]);
		ctrl.push_back(method);
	} else if (isMatch(words[1], "RMCMIN")) {
		method.type = METHOD_RMCMIN;
		check_option(sz, 5, "RMCMIN");
		method.iter   = str2long(words[2]);
		method.iterMC = str2long(words[3]);
		method.scale  = str2double(words[4]);
		ctrl.push_back(method);
	} else {
		cerr << "ERROR: Optimization type \'" << words[1] << "\' not recognized.\n"
		     << "Valid methods are INIT, GRID, ONE, ONEEX, RANDOM and MCMIN.\n"; 
		exit(1);
	}
}

bool Method::isMatch(const string &word, const char *method)
{
	return !strcasecmp(word.c_str(), method);
}

double Method::str2double(const string &word)
{
	return strtod(word.c_str(), NULL);
}

int Method::str2long(const string &word)
{
	return strtol(word.c_str(), NULL, 0);
}

string Method::report(size_t nm) const
{
	ostringstream ret;
	if (ctrl[nm].type == METHOD_INIT){
		ret << "METHOD INIT";
	} else if (ctrl[nm].type == METHOD_GRID){
		ret << "METHOD GRID"
		    << fixed << ctrl[nm].iter;
	} else if (ctrl[nm].type == METHOD_ONE){
		ret << "METHOD ONE";
	} else if (ctrl[nm].type == METHOD_ONEEX){
		ret << "METHOD ONEEX";
	} else if (ctrl[nm].type == METHOD_RANDOM) {
		ret << "METHOD RANDOM "
		    << fixed << ctrl[nm].iter;
	} else if (ctrl[nm].type == METHOD_MCMIN) {
		ret << "METHOD MCMIN "
		    << ctrl[nm].iter << " " << ctrl[nm].scale;
	} else if (ctrl[nm].type == METHOD_RMCMIN) {
		ret << "METHOD RMCMIN " << ctrl[nm].iter << " "
		    << ctrl[nm].iterMC << " " << ctrl[nm].scale;
	}
	return ret.str();
}

size_t Method::size() const
{
	return ctrl.size();
}

int Method::Type(size_t nm) const
{
	return ctrl[nm].type;
}

int Method::Iter(size_t nm) const
{
	return ctrl[nm].iter;
}

int Method::IterMC(size_t nm) const
{
	return ctrl[nm].iterMC;
}

double Method::Scale(size_t nm) const
{
	return ctrl[nm].scale;
}

void Method::check_option(int size, int limit, const char *name) const
{
	if (size < limit) {
		cerr << "ERROR: insufficient options for METHOD " << name << endl;
		exit(1);
	}
}
