#include "Xmgr.h"

Xmgr::Xmgr() : fXmgr(false), fError(true) {}

void Xmgr::SetXmgr(bool flag)  { fXmgr  = flag; }
void Xmgr::SetError(bool flag) { fError = flag; }
bool Xmgr::isXmgr() const      { return fXmgr;   }
bool Xmgr::isError() const     { return fError;  }

void Xmgr::header(const string &type)
{
	add_XMGRcommand("@ VERSION 40102");
	add_XMGRcommand("@ CLEAR STRING");
	add_XMGRcommand("@ PAGE SIZE 595, 842"); 	// A4 portrait
//	add_XMGRcommand("@ PAGE SIZE 612, 792"); 	// letter portrait
	add_XMGRcommand("@ VIEW XMIN 0.10");
	add_XMGRcommand("@ VIEW XMAX 0.95");
	add_XMGRcommand("@ VIEW YMIN 0.55");
	add_XMGRcommand("@ VIEW YMAX 0.92");
	add_XMGRcommand("@ legend on");
	add_XMGRcommand("@ legend box off");
	add_XMGRcommand("@ legend box fill off");
	add_XMGRcommand("@ legend box color 1");
	add_XMGRcommand("@ legend box linewidth 1");
	add_XMGRcommand("@ legend box linestyle 1");
	add_XMGRcommand("@ legend x1 0.60");
	add_XMGRcommand("@ legend y1 0.90");
	add_XMGRcommand("@ legend char size 0.6");
	add_XMGRcommand("@ g0 on");
	add_XMGRcommand("@ g0 hidden false");
	add_XMGRcommand("@ with g0");
	if (type == "CT-CPMG") {
		add_XMGRcommand("@ XAXIS LABEL \"1/\\xt\\f{}\\sCP\\N (s\\S-1\\N)\"");
		add_XMGRcommand("@ YAXIS LABEL \"\\qR\\Q\\s2\\N\\Seff\\N (s\\S-1\\N)\"");
		add_XMGRcommand("@ XAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ YAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ XAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ YAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ XAXIS TICKLABEL PREC 0");
		add_XMGRcommand("@ YAXIS TICKLABEL PREC 0");
	} else if (type == "R1rho") {
		add_XMGRcommand("@ XAXIS LABEL \"\\xw\\f{}\\s1\\N/(2\\xp\\f{}) (s\\S-1\\N)\"");
		add_XMGRcommand("@ YAXIS LABEL \"\\qR\\Q\\s1\\xr\\f{}\\N\\Seff\\N (s\\S-1\\N)\"");
		add_XMGRcommand("@ XAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ YAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ XAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ YAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ XAXIS TICKLABEL PREC 0");
		add_XMGRcommand("@ YAXIS TICKLABEL PREC 0");
	} else if (type == "R1rho_off") {
		add_XMGRcommand("@ XAXIS LABEL \"\\xW\\f{}/(2\\xp\\f{}) (Hz)\"");
		add_XMGRcommand("@ YAXIS LABEL \"\\qR\\Q\\s1\\xr\\f{}\\N\\Seff\\N (s\\S-1\\N)\"");
		add_XMGRcommand("@ XAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ YAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ XAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ YAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ XAXIS TICKLABEL PREC 0");
		add_XMGRcommand("@ YAXIS TICKLABEL PREC 0");
	} else if (type == "EXP") {
		add_XMGRcommand("@ XAXIS LABEL \"Delay (s)\"");
		add_XMGRcommand("@ YAXIS LABEL \"Intensity (a.u.)\"");
		add_XMGRcommand("@ XAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ YAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ XAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ YAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ XAXIS TICKLABEL PREC 1");
		add_XMGRcommand("@ YAXIS TICKLABEL PREC 1");
	} else if (type == "CLEANEX") {
		add_XMGRcommand("@ XAXIS LABEL \"Delay (s)\"");
		add_XMGRcommand("@ YAXIS LABEL \"I(t) / I\\s0\"");
		add_XMGRcommand("@ XAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ YAXIS TICKLABEL FORMAT DECIMAL");
		add_XMGRcommand("@ XAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ YAXIS TICKLABEL CHAR SIZE 0.8");
		add_XMGRcommand("@ XAXIS TICKLABEL PREC 2");
		add_XMGRcommand("@ YAXIS TICKLABEL PREC 1");
	}
}

void Xmgr::add_XMGRcommand(const char *line)
{
	command.push_back(line);
}

void Xmgr::add_plot(const string &label, const dvect &data, const ivect &index, double min
	, double max, int step, double slope)
{
	instr.push_back(Instr(label, data, index, min, max, step, slope));
}

void Xmgr::xplot(ostream &xout, const ParSet &pG, size_t ns, const ParDef &parm
	, const Funclib &func) const
{
	if (!instr.size()) {
		cout << "#WARNING: No plot instructions given\n";
		return;
	}

	// Write global options
	headerStyle(xout, parm.TITLE(ns), func.Eqn());

	// Print data
	double *ppar = NULL;
	Data plot_data;				// Temporal data
	int color = 0;				// Plot color
	int np = 0;				// Plot number
	int szInstr = instr.size();
	for (int i = 0; i < szInstr ; i++) {
		// Print plot style
		if (++color > 15) { color = 1; }
		if (!checkIndex(instr[i].idx, parm.indexBySet[ns])) { continue; }
		plotStyle(xout, np++, color, instr[i].label);
		double min =  DBL_MAX;		// Minimum X value
		double max = -DBL_MAX;		// Maximum X value
		int size = 101;			// Plot size
		bool bCopyMap = true;
		int sz = parm.nDataBySet[ns].size();
		for (int j = 0; j < sz; j++) {
			int nd = parm.nDataBySet[ns][j];	// Data number
			if (parm.data[nd].idx == instr[i].idx) {
				// Copy map. Do it once for each plot instruction and each dataset
				if (bCopyMap) {
					plot_data.map = parm.data[nd].map;
					bCopyMap = false;
				}

				// Plot X, Y (and Y-error) values
				xout << "\t" << parm.data[nd].x[0] << "\t" << parm.data[nd].y << "\t";
				if (fError) { xout << parm.data[nd].dy; }
				else        { xout << "1.0";            }
				xout << endl;

				// Search for minimum and maximum X values
				if (parm.data[nd].x[0] < min) { min = parm.data[nd].x[0]; }
				if (parm.data[nd].x[0] > max) { max = parm.data[nd].x[0]; }
			}
		}
		xout << "&\n";

		// Determine simulation limits
		if (instr[i].minPlot !=  DBL_MAX) { min  = instr[i].minPlot; }
		if (instr[i].maxPlot != -DBL_MAX) { max  = instr[i].maxPlot; }
		if (instr[i].nPlot)               { size = instr[i].nPlot;   }
		double step = (max - min) / (size - 1);

		// Construct the fake point
		plot_data.x    = instr[i].x;
		plot_data.x[0] = min;

		lineStyle(xout, np++, color, instr[i].label);

		dvect dy0;
		for (int j = 0; j < size; j++) {
			int sz = plot_data.map.size();
			make_new(ppar, sz, "xplot");
			for (int k = 0; k < sz; k++) {
				ppar[k] = pG.p[plot_data.map[k]];
			}
			double y = (*func.fx)(plot_data, ppar, dy0, bvect());
			zap(ppar);
			if (std::isfinite(y)) {
				xout << "\t" << plot_data.x[0] << "\t" << y << endl;
			}
			plot_data.x[0] += step;
			if (instr[i].slope != 1) {
				plot_data.x[1] += instr[i].slope;
			}
		}
		xout << "&\n";
	}
}

void Xmgr::annot_plot(ostream &xout, const ParSet &pG, size_t ns, const ParDef &parm
	, const dvect *err) const
{
	// Print parameters
	defaultString(xout);
	double xpos = 0.03;
	double ypos = 0.45;
	double space = 0.02;
	xout << "@ STRING FONT \"Courier-Bold\"\n";
	xout << "@ STRING " << xpos << ", " << ypos << endl;
	xout.setf(std::ios::left);		// Left justify
	xout << "@ STRING DEF \"Parameter   Fit       (Error)\"\n";
	xout.unsetf(std::ios::left);		// End left justify
	xout << "@ STRING on\n";

	ypos -= space * 0.3;
	int sz = parm.nParBySet[ns].size();
	for (int i = 0; i < sz; i++) {
		ypos -= space;
		int np = parm.nParBySet[ns][i];
		defaultString(xout);
		xout << "@ STRING " << xpos << ", " << ypos << endl;
		xout.setf(std::ios::left);	// Left justify
		xout << "@ STRING DEF \"";
		xout << setw(12) << parm.Name(np) << setw(10) << setprecision(5) << pG.p[np];
		if  (!parm.isFIXED(np) && err != NULL) {
			xout << "(" << setprecision(5) << (*err)[np] << ")";
		}
		xout << "\"\n";
		xout.unsetf(std::ios::left);	// End left justify
		xout << "@ STRING on\n";
	}

	// Print chi2
	defaultString(xout);
	xout << "@ STRING " << xpos << ", " << ypos - space << endl;
	xout << "@ STRING DEF \"";
	xout.setf(std::ios::left);
	xout << setw(12) << "X2/DoF" << setw(10) << setprecision(5)
	     << pG.lchi[ns] / parm.lDoF(ns) << "\"\n";
	xout.unsetf(std::ios::left);
	xout << "@ STRING on\n";
}

void Xmgr::annot_sims(ostream &xout, const ParSet &pM, const ParSet &pS, size_t ns
	, const ParDef &parm, int nsim, const string &label) const
{
	// Print parameters
	defaultString(xout);
	double xpos = 0.45 + 0.28 * (nsim - 1);
	double ypos = 0.45;
	double space = 0.02;
	xout << "@ STRING FONT \"Courier-Bold\"\n";
	xout << "@ STRING " << xpos << ", " << ypos << endl;
	xout.setf(std::ios::left);		// Left justify
	xout << "@ STRING DEF \"" << label << " (Error)\"\n";
	xout.unsetf(std::ios::left);		// End left justify
	xout << "@ STRING on\n";

	ypos -= space * 0.3;
	int sz = parm.nParBySet[ns].size();
	for (int i = 0; i < sz; i++) {
		int np = parm.nParBySet[ns][i];	// Parameter number

		ypos -= space;
		defaultString(xout);
		xout << "@ STRING " << xpos << ", " << ypos << endl;
		xout.setf(std::ios::left);	// Left justify
		xout << "@ STRING DEF \"";
		xout << setw(10) << setprecision(5) << pM.p[np] << "(";
		xout << setprecision(5) << pS.p[np] << ")\"\n";
		xout.unsetf(std::ios::left);	// End left justify
		xout << "@ STRING on\n";
	}

	// Print chi2
	defaultString(xout);
	xout << "@ STRING " << xpos << ", " << ypos - space << endl;
	xout << "@ STRING DEF \"";
	xout.setf(std::ios::left);
	xout << setw(10) << setprecision(5) << pM.lchi[ns] / parm.lDoF(ns) << "(";
	xout << setprecision(5) << pS.lchi[ns] / parm.lDoF(ns) << ")\"\n";
	xout.unsetf(std::ios::left);
	xout << "@ STRING on\n";
}

void Xmgr::headerStyle(ostream &xout, const string &title, const string &eqn) const
{
	int sz = command.size();
	for (int i = 0; i < sz ; i++) {
		xout << command[i] << "\n";
	}
	xout << "\n@ TITLE \"" << title << "\"\n" << "@ SUBTITLE \"" << eqn << "\"\n";
}

void Xmgr::plotStyle(ostream &xout, int np, int color, const string &label) const
{
	xout << "@ S"              << np << " LINESTYLE 0"              << endl;
	xout << "@ S"              << np << " COLOR " << color          << endl;
	xout << "@ S"              << np << " SYMBOL 2"                 << endl;
	xout << "@ S"              << np << " SYMBOL SIZE 0.5"          << endl;
	xout << "@ S"              << np << " SYMBOL FILL 1"            << endl;
	xout << "@ S"              << np << " SYMBOL COLOR " << color   << endl;
	xout << "@ S"              << np << " ERRORBAR LENGTH " << 0.5  << endl;
	if (!fError) {
		xout << "@ S"              << np << " ERRORBAR LINESTYLE 0"       << endl;
		xout << "@ S"              << np << " ERRORBAR RISER LINESTYLE 0" << endl;
	}
	xout << "@ legend string " << np << " \"Data " << label << "\"" << endl;
	xout << "@ TARGET S"       << np << endl;
	xout << "@ TYPE xydy" << endl;
}

void Xmgr::lineStyle(ostream &xout, int np, int color, const string &label) const
{
	xout << "@ S"              << np << " LINESTYLE 1"             << endl;
	xout << "@ S"              << np << " COLOR " << color         << endl;
	xout << "@ S"              << np << " SYMBOL 0"                << endl;
	xout << "@ legend string " << np << " \"Fit " << label << "\"" << endl;
	xout << "@ TARGET S"       << np << endl;
	xout << "@ TYPE xy" << endl;
}

void Xmgr::defaultString(ostream &xout) const
{
	xout << "@ WITH STRING\n";
	xout << "@ STRING FONT \"Courier\"\n";
	xout << "@ STRING LOCTYPE VIEW\n";
	xout << "@ STRING CHAR SIZE 0.6\n";
}

bool Xmgr::checkIndex(const ivect &plot_index, const i2vect &parm_index) const
{
	int szi = parm_index.size();
	for (int ni = 0; ni < szi; ni++) {
		if (parm_index[ni] == plot_index) {
			return true;
		}
	}
	return false;
}
